@@ -2,11 +2,9 @@
 import os
 import sys
 import inspect
+import types
 
-from doit.util import isgenerator
 from doit.task import InvalidTask, Task, dict_to_task
-from doit import dependency
-from doit import runner
 
 class InvalidCommand(Exception):
     """Invalid command line argument."""
@@ -20,22 +18,45 @@ class InvalidDodoFile(Exception):
 # that are task generators in a dodo file.
 TASK_STRING = "task_"
 
-def get_module(dodoFile):
+
+def isgenerator(object):
+    """Check if object type is a generator.
+
+    @param object: object to test.
+    @return: (bool) object is a generator?"""
+    return type(object) is types.GeneratorType
+
+
+def get_module(dodoFile, cwd=None):
     """
-    @param dodoFile: (string) path to file containing the tasks
+    @param dodoFile(str): path to file containing the tasks
+    @param cwd(str): path to be used cwd, if None use path from dodoFile
     @return (module) dodo module
     """
     ## load module dodo file and set environment
     base_path, file_name = os.path.split(os.path.abspath(dodoFile))
-    # make sure dir is on sys.path so we can import it
+    # make sure dodo path is on sys.path so we can import it
     sys.path.insert(0, base_path)
-    # file specified on dodo file are relative to itself.
-    os.chdir(base_path)
-    if not os.path.exists(file_name):
+
+    if cwd is None:
+        # by default cwd is same as dodo.py base path
+        full_cwd = base_path
+    else:
+        # insert specified cwd into sys.path
+        full_cwd = os.path.abspath(cwd)
+        if not os.path.isdir(full_cwd):
+            msg = "Specified 'dir' path must be a directory.\nGot '%s'(%s)."
+            raise InvalidCommand(msg % (cwd, full_cwd))
+        sys.path.insert(0, full_cwd)
+
+    if not os.path.exists(dodoFile):
         msg = ("Could not find dodo file '%s'.\n" +
-               "Please use '-f' to specify file name.\n" +
-               "Use the command 'dodo-sample' to view a sample dodo file.\n")
+               "Please use '-f' to specify file name.\n")
         raise InvalidDodoFile(msg % dodoFile)
+
+    # file specified on dodo file are relative to cwd
+    os.chdir(full_cwd)
+
     # get module containing the tasks
     return __import__(os.path.splitext(file_name)[0])
 
@@ -145,7 +166,7 @@ class TaskSetup(object):
     """
     Process dependencies and targets to find out the order tasks
     should be executed. Also apply filter to exclude tasks from
-    execution.
+    execution. And parse task cmd line options.
 
     @ivar filter: (sequence of strings) selection of tasks to execute
     @ivar tasks: (dict) Key: task name ([taskgen.]name)
@@ -154,9 +175,8 @@ class TaskSetup(object):
                           Value: L{Task} instance
     """
 
-    def __init__(self, task_list, filter_=None):
+    def __init__(self, task_list, cmd_opt=None):
 
-        self.filter = filter_
         self.targets = {}
         # name of task in order to be executed
         # this the order as in the dodo file. the real execution
@@ -179,6 +199,23 @@ class TaskSetup(object):
             self.tasks[task.name] = task
             self.task_order.append(task.name)
 
+        # process cmd line task options
+        # [task_name [-task_opt [opt_value]] ...] ...
+        self.filter = None
+        if cmd_opt is not None:
+            self.filter = []
+            seq = cmd_opt[:]
+            # process cmd_opts until nothing left
+            while seq:
+                f_name = seq.pop(0) # always start with a task/target name
+                self.filter.append(f_name)
+                if f_name not in self.tasks:
+                    continue # target specified, dont accept opts
+                # parse cmd_opt
+                the_task = self.tasks[f_name]
+                # remaining items are other tasks not positional options
+                the_task.options, seq = the_task.taskcmd.parse(seq)
+
 
         # check task-dependencies exist.
         for task in self.tasks.itervalues():
@@ -208,6 +245,7 @@ class TaskSetup(object):
                     task.task_dep.append(self.targets[dep].name)
 
 
+
     def _filter_tasks(self):
         """Select tasks specified by filter.
 
@@ -272,76 +310,3 @@ class TaskSetup(object):
         if self.filter is not None:
             selectedTask = self._filter_tasks()
         return self._order_tasks(selectedTask)
-
-
-
-##################################
-
-def doit_run(dependencyFile, task_list, filter_=None,
-             verbosity=0, alwaysExecute=False):
-    selected_tasks = TaskSetup(task_list, filter_).process()
-    return runner.run_tasks(dependencyFile, selected_tasks,
-                            verbosity, alwaysExecute)
-
-
-def doit_clean(task_list, clean_tasks):
-    """Clean tasks
-    @param task_list (list - L{Task}): list of all tasks from dodo file
-    @param clean_tasks (list - string): tasks bo be clean. clean all if
-                                        empty list.
-    """
-    if not clean_tasks:
-        for task_ in task_list:
-            task_.clean()
-    else:
-        tasks = dict([(t.name, t) for t in task_list])
-        for name in clean_tasks:
-            tasks[name].clean()
-
-
-def doit_list(task_list, printSubtasks, quiet=False):
-    """List task generators, in the order they were defined.
-
-    @param printSubtasks: (bool) print subtasks
-    """
-    for task in task_list:
-        if (not task.is_subtask) or printSubtasks:
-            task_str = task.name
-            if not quiet and task.doc:
-                task_str += " : %s" % task.doc
-            print task_str
-    return 0
-
-
-def doit_forget(dbFileName, taskList, forgetTasks):
-    """remove saved data successful runs from DB
-    @param dbFileName: (str)
-    @param task_list: (Task) tasks from dodo file
-    @param forget_tasks: (list - str) tasks to be removed. remove all if
-                         empty list.
-    """
-    dependencyManager = dependency.Dependency(dbFileName)
-    # no task specified. forget all
-    if not forgetTasks:
-        dependencyManager.remove_all()
-        print "forgeting all tasks"
-    # forget tasks from list
-    else:
-        tasks = dict([(t.name, t) for t in taskList])
-        for taskName in forgetTasks:
-            # check task exist
-            if taskName not in tasks:
-                msg = "'%s' is not a task."
-                raise InvalidCommand(msg % taskName)
-            # for group tasks also remove all tasks from group.
-            group = [taskName]
-            while group:
-                to_forget = group.pop(0)
-                if not tasks[to_forget].actions:
-                    # get task dependencies only from group-task
-                    group.extend(tasks[to_forget].task_dep)
-                # forget it - remove from dependency file
-                dependencyManager.remove(to_forget)
-                print "forgeting %s" % to_forget
-    dependencyManager.close()
-
