@@ -1,28 +1,18 @@
 """Task and actions classes."""
 import subprocess, sys
 import StringIO
-import traceback
 import inspect
 import os
+from threading import Thread
 
-from doit import logger
-
+from doit import TaskFailed, TaskError
+from doit import cmdparse
 
 # Exceptions
 class InvalidTask(Exception):
     """Invalid task instance. User error on specifying the task."""
     pass
 
-# TODO rename this?
-class TaskFailed(Exception):
-    """Task execution was not successful."""
-    pass
-
-class TaskError(Exception):
-    """Error while trying to execute task."""
-    pass
-
-
 
 
 
@@ -31,7 +21,7 @@ class BaseAction(object):
     """Base class for all actions"""
 
     # must implement:
-    # def execute(self, capture_stdout=False, capture_stderr=False)
+    # def execute(self, out=None, err=None)
 
     pass
 
@@ -51,41 +41,56 @@ class CmdAction(BaseAction):
         assert isinstance(action,str), "CmdAction must be a string."
         self.action = action
         self.task = None
+        self.out = None
+        self.err = None
+        self.value = None
 
-
-    def execute(self, capture_stdout=False, capture_stderr=False):
+    def execute(self, out=None, err=None):
         """
         Execute command action
 
-        @param capture_stdout(bool): Capture standard output
-        @param capture_err(bool): Capture standard error
+        both stdout and stderr from the command are captured and saved
+        on self.out/err. Real time output is controlled by parameters
+        @param out: None - no real time output
+                    a file like object (has write method)
+        @param err: idem
 
         @raise TaskError: If subprocess return code is greater than 125
         @raise TaskFailed: If subprocess return code isn't zero (and
         not greater than 125)
         """
-        # set Popen stream parameters
-        if not capture_stdout:
-            stdout = None
-        else:
-            stdout = subprocess.PIPE
-        if not capture_stderr:
-            stderr = None
-        else:
-            stderr = subprocess.PIPE
-
         action = self.expand_action()
 
         # spawn task process
-        process = subprocess.Popen(action,stdout=stdout,
-                                   stderr=stderr, shell=True)
+        process = subprocess.Popen(action, shell=True,
+                             stdout=subprocess.PIPE, stderr=subprocess.PIPE)
 
-        # log captured stream
-        out,err = process.communicate()
-        if out:
-            logger.log('stdout',out)
-        if err:
-            logger.log('stderr',err)
+        def print_process_output(process, input, capture, realtime):
+            while True:
+                # line buffered
+                line = input.readline()
+                # unbuffered ? process.stdout.read(1)
+                if line:
+                    capture.write(line)
+                    if realtime:
+                        realtime.write(line)
+                if not line and process.poll() != None:
+                    break
+
+        output = StringIO.StringIO()
+        errput = StringIO.StringIO()
+        t_out = Thread(target=print_process_output,
+                       args=(process, process.stdout, output, out))
+        t_err = Thread(target=print_process_output,
+                       args=(process, process.stderr, errput, err))
+        t_out.start()
+        t_err.start()
+        t_out.join()
+        t_err.join()
+
+        self.out = output.getvalue()
+        self.err = errput.getvalue()
+        self.value = self.out + self.err
 
         # task error - based on:
         # http://www.gnu.org/software/bash/manual/bashref.html#Exit-Status
@@ -99,6 +104,7 @@ class CmdAction(BaseAction):
             raise TaskFailed("Command failed: '%s' returned %s" %
                              (action,process.returncode))
 
+
     def expand_action(self):
         """expand action string using task meta informations
         @returns (string) - expanded string after substitution
@@ -111,6 +117,8 @@ class CmdAction(BaseAction):
         # just included changed if it is set
         if self.task.dep_changed is not None:
             subs_dict['changed'] = " ".join(self.task.dep_changed)
+        # task option parameters
+        subs_dict.update(self.task.options)
         return self.action % subs_dict
 
 
@@ -121,6 +129,18 @@ class CmdAction(BaseAction):
         return "<CmdAction: '%s'>"  % self.expand_action()
 
 
+
+
+class Writer(object):
+    """write to many streams"""
+    def __init__(self, *writers) :
+        self.writers = writers
+
+    def write(self, text) :
+        for w in self.writers :
+                w.write(text)
+
+
 class PythonAction(BaseAction):
     """Python action. Execute a python callable.
 
@@ -134,6 +154,9 @@ class PythonAction(BaseAction):
 
         self.py_callable = py_callable
         self.task = None
+        self.out = None
+        self.err = None
+        self.value = None
 
         if args is None:
             self.args = []
@@ -146,7 +169,7 @@ class PythonAction(BaseAction):
             self.kwargs = kwargs
 
         # check valid parameters
-        if not callable(self.py_callable):
+        if not hasattr(self.py_callable, '__call__'):
             raise InvalidTask("PythonAction must be a 'callable'.")
         if type(self.args) is not tuple and type(self.args) is not list:
             msg = "args must be a 'tuple' or a 'list'. got '%s'."
@@ -158,7 +181,8 @@ class PythonAction(BaseAction):
 
     def _prepare_kwargs(self):
         """
-        Prepare keyword arguments (targets, dependencies, changed)
+        Prepare keyword arguments (targets, dependencies, changed,
+        cmd line options)
         Inspect python callable and add missing arguments:
         - that the callable expects
         - have not been passed (as a regular arg or as keyword arg)
@@ -178,6 +202,9 @@ class PythonAction(BaseAction):
         extra_args = {'targets': self.task.targets,
                       'dependencies': self.task.file_dep,
                       'changed': self.task.dep_changed}
+
+        # tasks parameter options
+        extra_args.update(self.task.options)
         kwargs = self.kwargs.copy()
 
         for key in extra_args.keys():
@@ -202,27 +229,35 @@ class PythonAction(BaseAction):
             # if function has **kwargs include extra_arg on it
             elif argspec_keywords and key not in self.kwargs:
                 kwargs[key] = extra_args[key]
-
         return kwargs
 
 
-    def execute(self, capture_stdout=False, capture_stderr=False):
-        """
-        Execute command action
+    def execute(self, out=None, err=None):
+        """Execute command action
 
-        @param capture_stdout (bool): Capture standard output
-        @param capture_err (bool): Capture standard error
+        both stdout and stderr from the command are captured and saved
+        on self.out/err. Real time output is controlled by parameters
+        @param out: None - no real time output
+                    a file like object (has write method)
+        @param err: idem
 
         @raise TaskFailed: If py_callable returns False
         """
         # set std stream
-        if capture_stdout:
-            old_stdout = sys.stdout
-            sys.stdout = StringIO.StringIO()
+        old_stdout = sys.stdout
+        output = StringIO.StringIO()
+        old_stderr = sys.stderr
+        errput = StringIO.StringIO()
 
-        if capture_stderr:
-            old_stderr = sys.stderr
-            sys.stderr = StringIO.StringIO()
+        out_list = [output]
+        if out:
+            out_list.append(out)
+        err_list = [errput]
+        if err:
+            err_list.append(err)
+
+        sys.stdout = Writer(*out_list)
+        sys.stderr = Writer(*err_list)
 
         kwargs = self._prepare_kwargs()
 
@@ -233,39 +268,30 @@ class PythonAction(BaseAction):
                 result = self.py_callable(*self.args,**kwargs)
             # in python 2.4 SystemExit and KeyboardInterrupt subclass
             # from Exception.
-            except (SystemExit, KeyboardInterrupt), exp:
+            except (SystemExit, KeyboardInterrupt), exception:
                 raise
-
             except Exception, exception:
-                error = TaskError(exception)
-                error.originalException = traceback.format_exception(\
-                    exception.__class__, exception,sys.exc_info()[2])
-                raise error
+                raise TaskError("PythonAction Error", exception)
         finally:
             # restore std streams /log captured streams
-            if capture_stdout:
-                logger.log('stdout',sys.stdout.getvalue())
-                sys.stdout.close()
-                sys.stdout = old_stdout
-
-            if capture_stderr:
-                logger.log('stderr',sys.stderr.getvalue())
-                sys.stderr.close()
-                sys.stderr = old_stderr
-
-        # DEPREATED on 0.4 - to be removed on 0.5
-        if not isinstance(result, bool):
-            msg = "DEPREACTION WARNING: PythonAction must return a boolean."
-            if self.task:
-                msg += "Task %s." % self.task.name
-            msg += "Function '%s' returned %s"
-            print  msg % (self.py_callable.__name__, type(result))
-        # DEPRECATED - END
+            sys.stdout = old_stdout
+            sys.stderr = old_stderr
+            self.out = output.getvalue()
+            self.err = errput.getvalue()
 
+        self.value = result
         # if callable returns false. Task failed
-        if not result:
+        if result is False:
             raise TaskFailed("Python Task failed: '%s' returned %s" %
                              (self.py_callable, result))
+        elif result is True or result is None or isinstance(result, str):
+            return
+        else:
+            raise TaskError("Python Task error: '%s'. It must return:\n"
+                            "False for failed task.\n"
+                            "True, None or string for successful task\n"
+                            "returned %s (%s)" %
+                            (self.py_callable, result, type(result)))
 
     def __str__(self):
         # get object description excluding runtime memory address
@@ -292,7 +318,7 @@ def create_action(action):
     if type(action) is tuple:
         return PythonAction(*action)
 
-    if callable(action):
+    if hasattr(action, '__call__'):
         return PythonAction(action)
 
     msg = "Invalid task action type. got %s"
@@ -308,7 +334,6 @@ class Task(object):
     @ivar actions: list - L{BaseAction}
     @ivar clean_actions: list - L{BaseAction}
     @ivar targets: (list -string)
-    @ivar folder_dep: (list - string)
     @ivar task_dep: (list - string)
     @ivar file_dep: (list - string)
     @ivar dep_changed (list - string): list of file-dependencies that changed
@@ -318,69 +343,76 @@ class Task(object):
           (any object with setup or cleanup method)
     @ivar is_subtask: (bool) indicate this task is a subtask.
     @ivar doc: (string) task documentation
+
+    @ivar options: (dict) calculate params values
+    @ivar taskopt: (cmdparse.Command)
+    @ivar custom_title: function reference that takes a task object as
+                        parameter and returns a string.
     """
 
+    DEFAULT_VERBOSITY = 1
+
+    # list of valid types/values for each task attribute.
+    valid_attr = {'name': [str],
+                  'actions': [list, tuple, None],
+                  'dependencies': [list, tuple],
+                  'targets': [list, tuple],
+                  'setup': [list, tuple],
+                  'clean': [list, tuple, True],
+                  'doc': [str, None],
+                  'params': [list, tuple],
+                  'verbosity': [None,0,1,2],
+                  }
+
+
     def __init__(self, name, actions, dependencies=(), targets=(),
-                 setup=(), clean=(), is_subtask=False, doc=None):
-        """sanity checks and initialization"""
-
-        # dependencies parameter must be a list
-        if not ((isinstance(dependencies,list)) or
-                (isinstance(dependencies,tuple))):
-            msg = ("%s. paramater 'dependencies' must be a list or " +
-                   "tuple got:'%s'%s")
-            raise InvalidTask(msg%(name, str(dependencies),type(dependencies)))
-
-        # targets parameter must be a list
-        if not(isinstance(targets,list) or isinstance(targets,tuple)):
-            msg = ("%s. paramater 'targets' must be a list or tuple " +
-                   "got:'%s'%s")
-            raise InvalidTask(msg % (name, str(targets),type(targets)))
-
-        # setup parameter must be a list
-        if not(isinstance(setup,list) or isinstance(setup,tuple)):
-            ## DEPREACTED ON 0.4 - REMOVE ON 0.5
-            setup = [setup]
-            msg = "DEPRECATION WARNING: %s 'setup' must be a sequence."
-            print msg % name
+                 setup=(), clean=(), is_subtask=False, doc=None, params=(),
+                 verbosity=None, title=None):
+        """sanity checks and initialization
 
+        @param params: (list of option parameters) see cmdparse.Command.__init__
+        """
 
+        # check task attributes input
+        for attr, valid_list in self.valid_attr.iteritems():
+            self.check_attr_input(name, attr, locals()[attr], valid_list)
 
         self.name = name
         self.targets = targets
         self.setup = setup
         self.run_once = False
         self.is_subtask = is_subtask
+        self.value = None #TODO document this
+        self.verbosity = verbosity
+        self.custom_title = title
+
+        # options
+        self.taskcmd = cmdparse.TaskOption(name, params, None, None)
+        # put default values on options. this will be overwritten, if params
+        # options were passed on the command line.
+        self.options = self.taskcmd.parse('')[0] # ignore positional parameters
 
         if actions is None:
             self.actions = []
-        elif type(actions) is list:
-            self.actions = [create_action(a) for a in actions]
         else:
-        ## DEPREACTED ON 0.4 - REMOVE ON 0.5
-            #raise Exception("DEPRECATED must be list")
-            msg = "DEPRECATION WARNING: task %s actions must be a list."
-            print msg % self.name
-            self.actions = [create_action(actions)]
-        ## DEPRECATION END
+            self.actions = [create_action(a) for a in actions]
 
-        if clean == True:
+        if clean is True:
             self._remove_targets = True
             self.clean_actions = ()
         else:
             self._remove_targets = False
             self.clean_actions = [create_action(a) for a in clean]
 
-
         # set self as task for all actions
         for action in self.actions:
             action.task = self
 
         self.dep_changed = None
-        # there are 3 kinds of dependencies: file, task, and folder
-        self.folder_dep = []
+        # there are 2 kinds of dependencies: file, task
         self.task_dep = []
         self.file_dep = []
+        self.result_dep = []
         for dep in dependencies:
             # True on the list. set run_once
             if isinstance(dep,bool):
@@ -389,24 +421,22 @@ class Task(object):
                            "must be True got:'%s'")
                     raise InvalidTask(msg%(name, str(dep)))
                 self.run_once = True
-            # folder dep ends with a '/'
-            elif dep.endswith('/'):
-                self.folder_dep.append(dep)
             # task dep starts with a ':'
             elif dep.startswith(':'):
                 self.task_dep.append(dep[1:])
+            # task-result dep starts with a '?'
+            elif dep.startswith('?'):
+                self.result_dep.append(dep[1:])
             # file dep
             elif isinstance(dep,str):
                 self.file_dep.append(dep)
 
-
         # run_once can't be used together with file dependencies
         if self.run_once and self.file_dep:
             msg = ("%s. task cant have file and dependencies and True " +
                    "at the same time. (just remove True)")
             raise InvalidTask(msg % name)
 
-
         # Store just first non-empty line as documentation string
         if doc is None:
             self.doc = ''
@@ -420,37 +450,85 @@ class Task(object):
                 self.doc = ''
 
 
-    def execute(self, capture_stdout=False, capture_stderr=False):
+    @staticmethod
+    def check_attr_input(task, attr, value, valid):
+        """check input task attribute is correct type/value
+
+        @param task (string): task name
+        @param attr (string): attribute name
+        @param value: actual input from user
+        @param valid (list): of valid types/value accepted
+        @raises InvalidTask if invalid input
+        """
+        msg = "Task %s attribute '%s' must be {%s} got:%r %s"
+        for expected in valid:
+            # check expected type
+            if isinstance(expected, type):
+                if isinstance(value, expected):
+                    return
+            # check expected value
+            else:
+                if expected is value:
+                    return
+
+        # input value didnt match any valid type/value, raise execption
+        accept = ", ".join([getattr(v,'__name__',str(v)) for v in valid])
+        raise InvalidTask(msg % (task, attr, accept, str(value), type(value)))
+
+
+    def execute(self, out=None, err=None, verbosity=None):
         """Executes the task.
 
         @raise TaskFailed: If raised when executing an action
         @raise TaskError: If raised when executing an action
         """
+        # select verbosity to be used
+        priority = (verbosity, # use command line option
+                    self.verbosity, # or task default from dodo file
+                    self.DEFAULT_VERBOSITY) # or global default
+        use_verbosity = [v for v in  priority if v is not None][0]
+
+        VERBOSITY = [(None, None), # 0
+                     (None, err),  # 1
+                     (out, err)]   # 2
+        task_stdout, task_stderr = VERBOSITY[use_verbosity]
         for action in self.actions:
-            action.execute(capture_stdout, capture_stderr)
+            action.execute(task_stdout, task_stderr)
+            self.value = action.value
+
 
     def clean(self):
         """Execute task's clean"""
         # if clean is True remove all targets
         if self._remove_targets is True:
+            files = filter(os.path.isfile, self.targets)
+            dirs = filter(os.path.isdir, self.targets)
+
             # remove all files
-            for file_ in self.targets:
-                if not file_.endswith('/'):
-                    if os.path.exists(file_):
-                        os.remove(file_)
+            for file_ in files:
+                os.remove(file_)
+
+            # remove all directories (if empty)
+            for dir_ in dirs:
+                if os.listdir(dir_):
+                    msg = "Warning: '%s' cannot be removed (it is not empty)\n"
+                    sys.stderr.write(msg % dir_)
+                else:
+                    os.rmdir(dir_)
+
         else:
             # clean contains a list of actions...
             for action in self.clean_actions:
-                action.execute(False, False)
-
+                action.execute()
 
     def title(self):
         """String representation on output.
 
-        @rtype: str
-        @return: Task name and actions
+        @return: (str) Task name and actions
         """
-        return "%s => %s"% (self.name, str(self))
+        if self.custom_title:
+            return self.custom_title(self)
+        return self.name
 
 
     def __str__(self):
@@ -476,19 +554,9 @@ def dict_to_task(task_dict):
     """
     # TASK_ATTRS: sequence of know attributes(keys) of a task dict.
     TASK_ATTRS = ('name','actions','dependencies','targets','setup', 'doc',
-                  'clean')
+                  'clean', 'params', 'verbosity', 'title')
     # FIXME check field 'name'
 
-    # === DEPRECATED on 0.4 (to be removed on 0.5): START
-    if 'action' in task_dict and 'actions' not in task_dict:
-        #raise Exception("DEPRECATED action")
-        task_dict['actions'] = task_dict['action']
-        del task_dict['action']
-        print ("DEPRECATION WARNING: Task %s contains 'action' key. "
-               "This will be deprecated in future versions, "
-               "please use 'actions' instead" % task_dict['name'])
-    # === DEPRECATION: END
-
     # check required fields
     if 'actions' not in task_dict:
         raise InvalidTask("Task %s must contain 'actions' field. %s" %
@@ -497,7 +565,7 @@ def dict_to_task(task_dict):
     # user friendly. dont go ahead with invalid input.
     for key in task_dict.keys():
         if key not in TASK_ATTRS:
-            raise InvalidTask("Task %s contain invalid field: %s"%
+            raise InvalidTask("Task %s contains invalid field: '%s'"%
                               (task_dict['name'],key))
 
     return Task(**task_dict)
